; SPEED84a.ASM - Morse Code Speedometer using the 16F84 PIC
;
;
;  8/1/2000     Chuck Olson, WB9KZY
;               Jackson Harbor Press
;               http://jacksonharbor.home.att.net
;               jacksonharbor@att.net
;
;
;       What does it do?
;
;       This program uses a 16f84 to measure the "dit speed" of a Morse
;       code stream and then display the speed in wpm from 1 to 99 wpm
;       via either a 2 digit LED display or sent in Morse code to a piezo 
;       speaker.  In addition a "0.5" led was added to allow measurement
;       to the nearest .5 wpm.  Also, a piezo Morse code output was added,
;       when the pushbutton is pressed the element length ( multiply by 
;       10 us) is output in Morse through the piezo.
;
;
;       Possible Enhancements:
;
;       0) eliminate the hc595 shift registers for the LED display by using
;               PORTB for a multiplexed drive output of the 2 LEDs.
;       1) have an optional character or word speed mode, to more accurately
;               judge the speed when someone is "Farnsworthing".
;       2) add an LCD output on PORTB
;       3) change the measurement from the dit interval only, to the dit
;               interval PLUS the dit space interval - also measure the
;               dah interval in the same fashion - needed for "weighted"
;               sending, especially for manual sending.
;       4) add a decimal diagnostic mode to allow outputting the actual
;               interval length to the piezo in decimal rather than hex.
;       5) add a sleep mode (more for RFI quiet rather than for power
;               consumption)
;       6) add a display timeout to turn off the LEDs after a certain
;               period of inactivity
;       7) add some type of averaging or median filtering to prevent 
;               display "jumpiness"
;       8) add a weight display.  send a 2 digit percentage - the percentage
;               is the duty cycle of the weighted dit - an ideal 1:3 dit
;               to dah ratio will have a weight percentage of 50% (the dit
;               interval and the dit space interval are the same)
;       
;
;
;PIC
;pin    name    function/connection
;---    ----    -----------------------------------------------
;  1    ra2     key input - senses a dit/dah as logic 0, space as logic 1
;  2    ra3     strobe - pin 12 of the hc595
;  3    ra4     select - pin 11 of the hc595
;  4    MCLR    pulled up to +5V with a 10K resistor
;  5    VSS     Ground 
;  6    rb0     switch input - will send the speed through the piezo in Morse
;  7    rb1     no connect
;  8    rb2     no connect
;  9    rb3     .0 / .5 led output, led connected to +5V through a 510 ohm r
; 10    rb4     
; 11    rb5     
; 12    rb6     
; 13    rb7     
; 14    VDD     +5V
; 15    osc2    connected to 4 MHz xtal & 22? pf cap
; 16    osc1    connected to 4 MHz xtal & 22? pf cap
; 17    ra0     datao - pin 14 of the hc595
; 18    ra1     piezo speaker output for sending the speed in Morse


	LIST P=16F84, R=DEC 
	INCLUDE "p16F84.inc"
	__FUSES _PWRTE_OFF & _CP_OFF & _WDT_OFF & _XT_OSC
; powerup timer off - not sure why.
; copy protect is off
; watchdog timer is off 
; the oscillator uses an external 4 Mhx crystal or resonator



; memory locations  -  0C to 4F available in 16F84
tens    equ     H'0C'   ;holder for tens digit of converted speed byte
ones    equ     H'0D'   ;holder for ones digit of converted speed byte
temp    equ     H'0E'   ;all purpose temporary byte
highcb  equ     H'0F'   ;high key down counter byte
lowcb   equ     H'10'   ;low key down counter byte
char    equ     H'11'   ;holder for character data - used in playchr, strtbit
size    equ     H'12'   ;char size counter - used in playchr, strtbit
third   equ     H'13'   ;holder for number of dit length spaces => ditsp
temp2   equ     H'14'   ;holder for element length => outdit, outdah, ditsp
wpm     equ     H'15'   ;wpm counter - pointer to the length table => testlen
lastdhi equ     H'16'   ;last dit interval high byte
lastdlo equ     H'17'   ;last dit interval low byte
temp3   equ     H'18'   ;holder for element length / 2 - high byte
temp4   equ     H'19'   ;holder for element length / 2 - low byte
count   equ     H'1A'   ;count is used in bin2de2
R0      equ     H'1B'   ;used in bin2de2
R1      equ     H'1C'   ;used in bin2de2
R2      equ     H'1D'   ;used in bin2de2
hibyte  equ     H'1E'   ;scratch copy of highcb, used in bin2de2
lobyte  equ     H'1F'   ;scratch copy of lowcb, used in bin2de2                        


; constants - also see individual routines


; bit definitions
in      equ     2       ;bit 2 of port a is the input pin
carry   equ     0       ;bit 0 of STATUS is the carry bit
select  equ     3       ;select is pin 12 of the hc595 - bit 3 of PORTA
strobe  equ     4       ;strobe is pin 11 of the hc595 - bit 4 of PORTA
datao   equ     0       ;datao is pin 14 of the hc595 - bit 0 of PORTA
zero    equ     2       ;bit 2 of STATUS is the zero bit
switch  equ     0       ;bit 0 of PORTB is the switch input pin


	ORG     0x00
start   goto    init    ;startup routine
	

	ORG     0x04
rupt    goto    init    ;interrupt routine


; table of equivalent morse characters
;character structure from Dave Gwillym, KB2TQX
; right justified storage of the character:
;                                       dit = 0
;                                       dah = 1
; the leftmost 1 is a start bit - the byte is left shifted until a
;       1 is seen, then the dits and dahs are shifted out for a
;       total of 8 shifts - this holds most normal characters.
; note that the table position is what is stored in the memory
;
numbers addwf   PCL,1   ;compute the jump value
;                             character  hex      table position        
	retlw   b'00111111'  ;0 ascii   30 hex   00
	retlw   b'00101111'  ;1         31       01
	retlw   b'00100111'  ;2         32       02
	retlw   b'00100011'  ;3         33       03
	retlw   b'00100001'  ;4         34       04
	retlw   b'00100000'  ;5         35       05
	retlw   b'00110000'  ;6         36       06
	retlw   b'00111000'  ;7         37       07
	retlw   b'00111100'  ;8         38       08
	retlw   b'00111110'  ;9         39       09
;1/27/00 - eliminated the hex values below - they are only needed for
;       sending the raw data to the piezo in hex (sendit, maskns routines)
;lets    retlw   b'00000101'  ;A         3A hex   0A
;        retlw   b'00011000'  ;B         3B       0B
;        retlw   b'00011010'  ;C         3C       0C
;        retlw   b'00001100'  ;D         3D       0D
;        retlw   b'00000010'  ;E         3E       0E
;        retlw   b'00010010'  ;F         3F       0F


; sevseg - 10 bytes that define the segment data for 0-9 on the LED
;
;  bit  7  6  5  4  3  2  1  0
;  seg  a  b  c  d  e  f  g  -
;
;      +----a----+
;      |         |
;      f         b          1 = on
;      |         |          0 = off
;      +----g----+          common cathode
;      |         |
;      e         c
;      |         |
;      +----d----+  
sevseg  addwf   PCL,1   ;compute the jump value
;                 abcdefg
	retlw   b'11111100'   ;0
	retlw   b'01100000'   ;1
	retlw   b'11011010'   ;2
	retlw   b'11110010'   ;3
	retlw   b'01100110'   ;4
	retlw   b'10110110'   ;5
	retlw   b'10111110'   ;6
	retlw   b'11100000'   ;7
	retlw   b'11111110'   ;8
	retlw   b'11110110'   ;9
; hex display (A-F) not needed in this program
;        retlw   b'11101110'   ;A
;        retlw   b'00111110'   ;b
;        retlw   b'10011100'   ;C
;        retlw   b'01111010'   ;d
;        retlw   b'10011110'   ;E
;        retlw   b'10001110'   ;F
			  

; 1/28/00 commented out the following common anode version of the
;       seven segment table
;; sevseg - 10 bytes that define the segment data for 0-9 on the LED
;;
;;  bit  7  6  5  4  3  2  1  0
;;  seg  a  b  c  d  e  f  g  -
;;
;;      +----a----+
;;      |         |
;;      f         b          1 = on
;;      |         |          0 = off
;;      +----g----+          common anode
;;      |         |
;;      e         c
;;      |         |
;;      +----d----+  
;sevseg  addwf   PCL,1   ;compute the jump value
;;                 abcdefg
;        retlw   b'00000010'   ;0
;        retlw   b'10011110'   ;1
;        retlw   b'00100100'   ;2
;        retlw   b'00001100'   ;3
;        retlw   b'10011000'   ;4
;        retlw   b'01001000'   ;5
;        retlw   b'01000000'   ;6
;        retlw   b'00011110'   ;7
;        retlw   b'00000000'   ;8
;        retlw   b'00001000'   ;9
;; hex display (A-F) not needed in this program
;;        retlw   b'00010000'   ;A
;;        retlw   b'11000000'   ;b
;;        retlw   b'01100010'   ;C
;;        retlw   b'10000100'   ;d
;;        retlw   b'01100000'   ;E
;;        retlw   b'01110000'   ;F
			   

; length - a byte wide table of dit interval decision points.  It
;       was generated with the Qbasic program, speedtab4.bas - then the 
;       table was edited to get rid of the spaces around the decimal value.
;       The decision points are at half word per minute (wpm) intervals.
;       If the measured dit interval is greater than the present decision 
;       point value and less than the previous value, the speed is simply
;       the table pointer.  The decision points are the most significan byte.
;       The LSB is in the table length2.
length  addwf   PCL,1   ;compute the jump value
	retlw   d'110' ;  4.25 wpm    28235 x .01 ms - high
	retlw   d'98'  ;  4.75 wpm    25263 x .01 ms - high
	retlw   d'89'  ;  5.25 wpm    22857 x .01 ms - high
	retlw   d'81'  ;  5.75 wpm    20870 x .01 ms - high
	retlw   d'75'  ;  6.25 wpm    19200 x .01 ms - high
	retlw   d'69'  ;  6.75 wpm    17778 x .01 ms - high
	retlw   d'64'  ;  7.25 wpm    16552 x .01 ms - high
	retlw   d'60'  ;  7.75 wpm    15484 x .01 ms - high
	retlw   d'56'  ;  8.25 wpm    14545 x .01 ms - high
	retlw   d'53'  ;  8.75 wpm    13714 x .01 ms - high
	retlw   d'50'  ;  9.25 wpm    12973 x .01 ms - high
	retlw   d'48'  ;  9.75 wpm    12308 x .01 ms - high
	retlw   d'45'  ; 10.25 wpm    11707 x .01 ms - high
	retlw   d'43'  ; 10.75 wpm    11163 x .01 ms - high
	retlw   d'41'  ; 11.25 wpm    10667 x .01 ms - high
	retlw   d'39'  ; 11.75 wpm    10213 x .01 ms - high
	retlw   d'38'  ; 12.25 wpm     9796 x .01 ms - high
	retlw   d'36'  ; 12.75 wpm     9412 x .01 ms - high
	retlw   d'35'  ; 13.25 wpm     9057 x .01 ms - high
	retlw   d'34'  ; 13.75 wpm     8727 x .01 ms - high
	retlw   d'32'  ; 14.25 wpm     8421 x .01 ms - high
	retlw   d'31'  ; 14.75 wpm     8136 x .01 ms - high
	retlw   d'30'  ; 15.25 wpm     7869 x .01 ms - high
	retlw   d'29'  ; 15.75 wpm     7619 x .01 ms - high
	retlw   d'28'  ; 16.25 wpm     7385 x .01 ms - high
	retlw   d'27'  ; 16.75 wpm     7164 x .01 ms - high
	retlw   d'27'  ; 17.25 wpm     6957 x .01 ms - high
	retlw   d'26'  ; 17.75 wpm     6761 x .01 ms - high
	retlw   d'25'  ; 18.25 wpm     6575 x .01 ms - high
	retlw   d'25'  ; 18.75 wpm     6400 x .01 ms - high
	retlw   d'24'  ; 19.25 wpm     6234 x .01 ms - high
	retlw   d'23'  ; 19.75 wpm     6076 x .01 ms - high
	retlw   d'23'  ; 20.25 wpm     5926 x .01 ms - high
	retlw   d'22'  ; 20.75 wpm     5783 x .01 ms - high
	retlw   d'22'  ; 21.25 wpm     5647 x .01 ms - high
	retlw   d'21'  ; 21.75 wpm     5517 x .01 ms - high
	retlw   d'21'  ; 22.25 wpm     5393 x .01 ms - high
	retlw   d'20'  ; 22.75 wpm     5275 x .01 ms - high
	retlw   d'20'  ; 23.25 wpm     5161 x .01 ms - high
	retlw   d'19'  ; 23.75 wpm     5053 x .01 ms - high
	retlw   d'19'  ; 24.25 wpm     4948 x .01 ms - high
	retlw   d'18'  ; 24.75 wpm     4848 x .01 ms - high
	retlw   d'18'  ; 25.25 wpm     4752 x .01 ms - high
	retlw   d'18'  ; 25.75 wpm     4660 x .01 ms - high
	retlw   d'17'  ; 26.25 wpm     4571 x .01 ms - high
	retlw   d'17'  ; 26.75 wpm     4486 x .01 ms - high
	retlw   d'17'  ; 27.25 wpm     4404 x .01 ms - high
	retlw   d'16'  ; 27.75 wpm     4324 x .01 ms - high
	retlw   d'16'  ; 28.25 wpm     4248 x .01 ms - high
	retlw   d'16'  ; 28.75 wpm     4174 x .01 ms - high
	retlw   d'16'  ; 29.25 wpm     4103 x .01 ms - high
	retlw   d'15'  ; 29.75 wpm     4034 x .01 ms - high
	retlw   d'15'  ; 30.25 wpm     3967 x .01 ms - high
	retlw   d'15'  ; 30.75 wpm     3902 x .01 ms - high
	retlw   d'15'  ; 31.25 wpm     3840 x .01 ms - high
	retlw   d'14'  ; 31.75 wpm     3780 x .01 ms - high
	retlw   d'14'  ; 32.25 wpm     3721 x .01 ms - high
	retlw   d'14'  ; 32.75 wpm     3664 x .01 ms - high
	retlw   d'14'  ; 33.25 wpm     3609 x .01 ms - high
	retlw   d'13'  ; 33.75 wpm     3556 x .01 ms - high
	retlw   d'13'  ; 34.25 wpm     3504 x .01 ms - high
	retlw   d'13'  ; 34.75 wpm     3453 x .01 ms - high
	retlw   d'13'  ; 35.25 wpm     3404 x .01 ms - high
	retlw   d'13'  ; 35.75 wpm     3357 x .01 ms - high
	retlw   d'12'  ; 36.25 wpm     3310 x .01 ms - high
	retlw   d'12'  ; 36.75 wpm     3265 x .01 ms - high
	retlw   d'12'  ; 37.25 wpm     3221 x .01 ms - high
	retlw   d'12'  ; 37.75 wpm     3179 x .01 ms - high
	retlw   d'12'  ; 38.25 wpm     3137 x .01 ms - high
	retlw   d'12'  ; 38.75 wpm     3097 x .01 ms - high
	retlw   d'11'  ; 39.25 wpm     3057 x .01 ms - high
	retlw   d'11'  ; 39.75 wpm     3019 x .01 ms - high
	retlw   d'11'  ; 40.25 wpm     2981 x .01 ms - high
	retlw   d'11'  ; 40.75 wpm     2945 x .01 ms - high
	retlw   d'11'  ; 41.25 wpm     2909 x .01 ms - high
	retlw   d'11'  ; 41.75 wpm     2874 x .01 ms - high
	retlw   d'11'  ; 42.25 wpm     2840 x .01 ms - high
	retlw   d'10'  ; 42.75 wpm     2807 x .01 ms - high
	retlw   d'10'  ; 43.25 wpm     2775 x .01 ms - high
	retlw   d'10'  ; 43.75 wpm     2743 x .01 ms - high
	retlw   d'10'  ; 44.25 wpm     2712 x .01 ms - high
	retlw   d'10'  ; 44.75 wpm     2682 x .01 ms - high
	retlw   d'10'  ; 45.25 wpm     2652 x .01 ms - high
	retlw   d'10'  ; 45.75 wpm     2623 x .01 ms - high
	retlw   d'10'  ; 46.25 wpm     2595 x .01 ms - high
	retlw   d'10'  ; 46.75 wpm     2567 x .01 ms - high
	retlw   d'9'   ; 47.25 wpm     2540 x .01 ms - high
	retlw   d'9'   ; 47.75 wpm     2513 x .01 ms - high
	retlw   d'9'   ; 48.25 wpm     2487 x .01 ms - high
	retlw   d'9'   ; 48.75 wpm     2462 x .01 ms - high
	retlw   d'9'   ; 49.25 wpm     2437 x .01 ms - high
	retlw   d'9'   ; 49.75 wpm     2412 x .01 ms - high
	retlw   d'9'   ; 50.25 wpm     2388 x .01 ms - high
	retlw   d'9'   ; 50.75 wpm     2365 x .01 ms - high
	retlw   d'9'   ; 51.25 wpm     2341 x .01 ms - high
	retlw   d'9'   ; 51.75 wpm     2319 x .01 ms - high
	retlw   d'8'   ; 52.25 wpm     2297 x .01 ms - high
	retlw   d'8'   ; 52.75 wpm     2275 x .01 ms - high
	retlw   d'8'   ; 53.25 wpm     2254 x .01 ms - high
	retlw   d'8'   ; 53.75 wpm     2233 x .01 ms - high
	retlw   d'8'   ; 54.25 wpm     2212 x .01 ms - high
	retlw   d'8'   ; 54.75 wpm     2192 x .01 ms - high
	retlw   d'8'   ; 55.25 wpm     2172 x .01 ms - high
	retlw   d'8'   ; 55.75 wpm     2152 x .01 ms - high
	retlw   d'8'   ; 56.25 wpm     2133 x .01 ms - high
	retlw   d'8'   ; 56.75 wpm     2115 x .01 ms - high
	retlw   d'8'   ; 57.25 wpm     2096 x .01 ms - high
	retlw   d'8'   ; 57.75 wpm     2078 x .01 ms - high
	retlw   d'8'   ; 58.25 wpm     2060 x .01 ms - high
	retlw   d'7'   ; 58.75 wpm     2043 x .01 ms - high
	retlw   d'7'   ; 59.25 wpm     2025 x .01 ms - high
	retlw   d'7'   ; 59.75 wpm     2008 x .01 ms - high
	retlw   d'7'   ; 60.25 wpm     1992 x .01 ms - high
	retlw   d'7'   ; 60.75 wpm     1975 x .01 ms - high
	retlw   d'7'   ; 61.25 wpm     1959 x .01 ms - high
	retlw   d'7'   ; 61.75 wpm     1943 x .01 ms - high
	retlw   d'7'   ; 62.25 wpm     1928 x .01 ms - high
	retlw   d'7'   ; 62.75 wpm     1912 x .01 ms - high
	retlw   d'7'   ; 63.25 wpm     1897 x .01 ms - high
	retlw   d'7'   ; 63.75 wpm     1882 x .01 ms - high
	retlw   d'7'   ; 64.25 wpm     1868 x .01 ms - high
	retlw   d'7'   ; 64.75 wpm     1853 x .01 ms - high
	retlw   d'7'   ; 65.25 wpm     1839 x .01 ms - high
	retlw   d'7'   ; 65.75 wpm     1825 x .01 ms - high
	retlw   d'7'   ; 66.25 wpm     1811 x .01 ms - high
	retlw   d'7'   ; 66.75 wpm     1798 x .01 ms - high
	retlw   d'6'   ; 67.25 wpm     1784 x .01 ms - high
	retlw   d'6'   ; 67.75 wpm     1771 x .01 ms - high
	retlw   d'6'   ; 68.25 wpm     1758 x .01 ms - high
	retlw   d'6'   ; 68.75 wpm     1745 x .01 ms - high
	retlw   d'6'   ; 69.25 wpm     1733 x .01 ms - high
	retlw   d'6'   ; 69.75 wpm     1720 x .01 ms - high
	retlw   d'6'   ; 70.25 wpm     1708 x .01 ms - high
	retlw   d'6'   ; 70.75 wpm     1696 x .01 ms - high
	retlw   d'6'   ; 71.25 wpm     1684 x .01 ms - high
	retlw   d'6'   ; 71.75 wpm     1672 x .01 ms - high
	retlw   d'6'   ; 72.25 wpm     1661 x .01 ms - high
	retlw   d'6'   ; 72.75 wpm     1649 x .01 ms - high
	retlw   d'6'   ; 73.25 wpm     1638 x .01 ms - high
	retlw   d'6'   ; 73.75 wpm     1627 x .01 ms - high
	retlw   d'6'   ; 74.25 wpm     1616 x .01 ms - high
	retlw   d'6'   ; 74.75 wpm     1605 x .01 ms - high
	retlw   d'6'   ; 75.25 wpm     1595 x .01 ms - high
	retlw   d'6'   ; 75.75 wpm     1584 x .01 ms - high
	retlw   d'6'   ; 76.25 wpm     1574 x .01 ms - high
	retlw   d'6'   ; 76.75 wpm     1564 x .01 ms - high
	retlw   d'6'   ; 77.25 wpm     1553 x .01 ms - high
	retlw   d'6'   ; 77.75 wpm     1543 x .01 ms - high
	retlw   d'5'   ; 78.25 wpm     1534 x .01 ms - high
	retlw   d'5'   ; 78.75 wpm     1524 x .01 ms - high
	retlw   d'5'   ; 79.25 wpm     1514 x .01 ms - high
	retlw   d'5'   ; 79.75 wpm     1505 x .01 ms - high
	retlw   d'5'   ; 80.25 wpm     1495 x .01 ms - high
	retlw   d'5'   ; 80.75 wpm     1486 x .01 ms - high
	retlw   d'5'   ; 81.25 wpm     1477 x .01 ms - high
	retlw   d'5'   ; 81.75 wpm     1468 x .01 ms - high
	retlw   d'5'   ; 82.25 wpm     1459 x .01 ms - high
	retlw   d'5'   ; 82.75 wpm     1450 x .01 ms - high
	retlw   d'5'   ; 83.25 wpm     1441 x .01 ms - high
	retlw   d'5'   ; 83.75 wpm     1433 x .01 ms - high
	retlw   d'5'   ; 84.25 wpm     1424 x .01 ms - high
	retlw   d'5'   ; 84.75 wpm     1416 x .01 ms - high
	retlw   d'5'   ; 85.25 wpm     1408 x .01 ms - high
	retlw   d'5'   ; 85.75 wpm     1399 x .01 ms - high
	retlw   d'5'   ; 86.25 wpm     1391 x .01 ms - high
	retlw   d'5'   ; 86.75 wpm     1383 x .01 ms - high
	retlw   d'5'   ; 87.25 wpm     1375 x .01 ms - high
	retlw   d'5'   ; 87.75 wpm     1368 x .01 ms - high
	retlw   d'5'   ; 88.25 wpm     1360 x .01 ms - high
	retlw   d'5'   ; 88.75 wpm     1352 x .01 ms - high
	retlw   d'5'   ; 89.25 wpm     1345 x .01 ms - high
	retlw   d'5'   ; 89.75 wpm     1337 x .01 ms - high
	retlw   d'5'   ; 90.25 wpm     1330 x .01 ms - high
	retlw   d'5'   ; 90.75 wpm     1322 x .01 ms - high
	retlw   d'5'   ; 91.25 wpm     1315 x .01 ms - high
	retlw   d'5'   ; 91.75 wpm     1308 x .01 ms - high
	retlw   d'5'   ; 92.25 wpm     1301 x .01 ms - high
	retlw   d'5'   ; 92.75 wpm     1294 x .01 ms - high
	retlw   d'5'   ; 93.25 wpm     1287 x .01 ms - high
	retlw   d'5'   ; 93.75 wpm     1280 x .01 ms - high
	retlw   d'4'   ; 94.25 wpm     1273 x .01 ms - high
	retlw   d'4'   ; 94.75 wpm     1266 x .01 ms - high
	retlw   d'4'   ; 95.25 wpm     1260 x .01 ms - high
	retlw   d'4'   ; 95.75 wpm     1253 x .01 ms - high
	retlw   d'4'   ; 96.25 wpm     1247 x .01 ms - high
	retlw   d'4'   ; 96.75 wpm     1240 x .01 ms - high
	retlw   d'4'   ; 97.25 wpm     1234 x .01 ms - high
	retlw   d'4'   ; 97.75 wpm     1228 x .01 ms - high
	retlw   d'4'   ; 98.25 wpm     1221 x .01 ms - high
	retlw   d'4'   ; 98.75 wpm     1215 x .01 ms - high
	retlw   d'4'   ; 99.25 wpm     1209 x .01 ms - high
	retlw   d'4'   ; 99.75 wpm     1203 x .01 ms - high

	org     0x100
	
; length2 - a byte wide table of dit interval decision points.  It
;       was generated with the Qbasic program, speedtab4.bas - then the 
;       table was edited to get rid of the spaces around the decimal value.
;       The decision points are at half word per minute (wpm) intervals.
;       If the measured dit interval is greater than the present decision 
;       point value and less than the previous value, the speed is simply
;       the table pointer.  The decision points are the least significan byte.
;       The MSB is in the table length.
length2 addwf   PCL,1   ;compute the jump value
	retlw   d'75'    ;  4.25 wpm    28235 x .01 ms - low
	retlw   d'175'   ;  4.75 wpm    25263 x .01 ms - low
	retlw   d'73'    ;  5.25 wpm    22857 x .01 ms - low
	retlw   d'134'   ;  5.75 wpm    20870 x .01 ms - low
	retlw   d'0'     ;  6.25 wpm    19200 x .01 ms - low
	retlw   d'114'   ;  6.75 wpm    17778 x .01 ms - low
	retlw   d'168'   ;  7.25 wpm    16552 x .01 ms - low
	retlw   d'124'   ;  7.75 wpm    15484 x .01 ms - low
	retlw   d'209'   ;  8.25 wpm    14545 x .01 ms - low
	retlw   d'146'   ;  8.75 wpm    13714 x .01 ms - low
	retlw   d'173'   ;  9.25 wpm    12973 x .01 ms - low
	retlw   d'20'    ;  9.75 wpm    12308 x .01 ms - low
	retlw   d'187'   ; 10.25 wpm    11707 x .01 ms - low
	retlw   d'155'   ; 10.75 wpm    11163 x .01 ms - low
	retlw   d'171'   ; 11.25 wpm    10667 x .01 ms - low
	retlw   d'229'   ; 11.75 wpm    10213 x .01 ms - low
	retlw   d'68'    ; 12.25 wpm     9796 x .01 ms - low
	retlw   d'196'   ; 12.75 wpm     9412 x .01 ms - low
	retlw   d'97'    ; 13.25 wpm     9057 x .01 ms - low
	retlw   d'23'    ; 13.75 wpm     8727 x .01 ms - low
	retlw   d'229'   ; 14.25 wpm     8421 x .01 ms - low
	retlw   d'200'   ; 14.75 wpm     8136 x .01 ms - low
	retlw   d'189'   ; 15.25 wpm     7869 x .01 ms - low
	retlw   d'195'   ; 15.75 wpm     7619 x .01 ms - low
	retlw   d'217'   ; 16.25 wpm     7385 x .01 ms - low
	retlw   d'252'   ; 16.75 wpm     7164 x .01 ms - low
	retlw   d'45'    ; 17.25 wpm     6957 x .01 ms - low
	retlw   d'105'   ; 17.75 wpm     6761 x .01 ms - low
	retlw   d'175'   ; 18.25 wpm     6575 x .01 ms - low
	retlw   d'0'     ; 18.75 wpm     6400 x .01 ms - low
	retlw   d'90'    ; 19.25 wpm     6234 x .01 ms - low
	retlw   d'188'   ; 19.75 wpm     6076 x .01 ms - low
	retlw   d'38'    ; 20.25 wpm     5926 x .01 ms - low
	retlw   d'151'   ; 20.75 wpm     5783 x .01 ms - low
	retlw   d'15'    ; 21.25 wpm     5647 x .01 ms - low
	retlw   d'141'   ; 21.75 wpm     5517 x .01 ms - low
	retlw   d'17'    ; 22.25 wpm     5393 x .01 ms - low
	retlw   d'155'   ; 22.75 wpm     5275 x .01 ms - low
	retlw   d'41'    ; 23.25 wpm     5161 x .01 ms - low
	retlw   d'189'   ; 23.75 wpm     5053 x .01 ms - low
	retlw   d'84'    ; 24.25 wpm     4948 x .01 ms - low
	retlw   d'240'   ; 24.75 wpm     4848 x .01 ms - low
	retlw   d'144'   ; 25.25 wpm     4752 x .01 ms - low
	retlw   d'52'    ; 25.75 wpm     4660 x .01 ms - low
	retlw   d'219'   ; 26.25 wpm     4571 x .01 ms - low
	retlw   d'134'   ; 26.75 wpm     4486 x .01 ms - low
	retlw   d'52'    ; 27.25 wpm     4404 x .01 ms - low
	retlw   d'228'   ; 27.75 wpm     4324 x .01 ms - low
	retlw   d'152'   ; 28.25 wpm     4248 x .01 ms - low
	retlw   d'78'    ; 28.75 wpm     4174 x .01 ms - low
	retlw   d'7'     ; 29.25 wpm     4103 x .01 ms - low
	retlw   d'194'   ; 29.75 wpm     4034 x .01 ms - low
	retlw   d'127'   ; 30.25 wpm     3967 x .01 ms - low
	retlw   d'62'    ; 30.75 wpm     3902 x .01 ms - low
	retlw   d'0'     ; 31.25 wpm     3840 x .01 ms - low
	retlw   d'196'   ; 31.75 wpm     3780 x .01 ms - low
	retlw   d'137'   ; 32.25 wpm     3721 x .01 ms - low
	retlw   d'80'    ; 32.75 wpm     3664 x .01 ms - low
	retlw   d'25'    ; 33.25 wpm     3609 x .01 ms - low
	retlw   d'228'   ; 33.75 wpm     3556 x .01 ms - low
	retlw   d'176'   ; 34.25 wpm     3504 x .01 ms - low
	retlw   d'125'   ; 34.75 wpm     3453 x .01 ms - low
	retlw   d'76'    ; 35.25 wpm     3404 x .01 ms - low
	retlw   d'29'    ; 35.75 wpm     3357 x .01 ms - low
	retlw   d'238'   ; 36.25 wpm     3310 x .01 ms - low
	retlw   d'193'   ; 36.75 wpm     3265 x .01 ms - low
	retlw   d'149'   ; 37.25 wpm     3221 x .01 ms - low
	retlw   d'107'   ; 37.75 wpm     3179 x .01 ms - low
	retlw   d'65'    ; 38.25 wpm     3137 x .01 ms - low
	retlw   d'25'    ; 38.75 wpm     3097 x .01 ms - low
	retlw   d'241'   ; 39.25 wpm     3057 x .01 ms - low
	retlw   d'203'   ; 39.75 wpm     3019 x .01 ms - low
	retlw   d'165'   ; 40.25 wpm     2981 x .01 ms - low
	retlw   d'129'   ; 40.75 wpm     2945 x .01 ms - low
	retlw   d'93'    ; 41.25 wpm     2909 x .01 ms - low
	retlw   d'58'    ; 41.75 wpm     2874 x .01 ms - low
	retlw   d'24'    ; 42.25 wpm     2840 x .01 ms - low
	retlw   d'247'   ; 42.75 wpm     2807 x .01 ms - low
	retlw   d'215'   ; 43.25 wpm     2775 x .01 ms - low
	retlw   d'183'   ; 43.75 wpm     2743 x .01 ms - low
	retlw   d'152'   ; 44.25 wpm     2712 x .01 ms - low
	retlw   d'122'   ; 44.75 wpm     2682 x .01 ms - low
	retlw   d'92'    ; 45.25 wpm     2652 x .01 ms - low
	retlw   d'63'    ; 45.75 wpm     2623 x .01 ms - low
	retlw   d'35'    ; 46.25 wpm     2595 x .01 ms - low
	retlw   d'7'     ; 46.75 wpm     2567 x .01 ms - low
	retlw   d'236'   ; 47.25 wpm     2540 x .01 ms - low
	retlw   d'209'   ; 47.75 wpm     2513 x .01 ms - low
	retlw   d'183'   ; 48.25 wpm     2487 x .01 ms - low
	retlw   d'158'   ; 48.75 wpm     2462 x .01 ms - low
	retlw   d'133'   ; 49.25 wpm     2437 x .01 ms - low
	retlw   d'108'   ; 49.75 wpm     2412 x .01 ms - low
	retlw   d'84'    ; 50.25 wpm     2388 x .01 ms - low
	retlw   d'61'    ; 50.75 wpm     2365 x .01 ms - low
	retlw   d'37'    ; 51.25 wpm     2341 x .01 ms - low
	retlw   d'15'    ; 51.75 wpm     2319 x .01 ms - low
	retlw   d'249'   ; 52.25 wpm     2297 x .01 ms - low
	retlw   d'227'   ; 52.75 wpm     2275 x .01 ms - low
	retlw   d'206'   ; 53.25 wpm     2254 x .01 ms - low
	retlw   d'185'   ; 53.75 wpm     2233 x .01 ms - low
	retlw   d'164'   ; 54.25 wpm     2212 x .01 ms - low
	retlw   d'144'   ; 54.75 wpm     2192 x .01 ms - low
	retlw   d'124'   ; 55.25 wpm     2172 x .01 ms - low
	retlw   d'104'   ; 55.75 wpm     2152 x .01 ms - low
	retlw   d'85'    ; 56.25 wpm     2133 x .01 ms - low
	retlw   d'67'    ; 56.75 wpm     2115 x .01 ms - low
	retlw   d'48'    ; 57.25 wpm     2096 x .01 ms - low
	retlw   d'30'    ; 57.75 wpm     2078 x .01 ms - low
	retlw   d'12'    ; 58.25 wpm     2060 x .01 ms - low
	retlw   d'251'   ; 58.75 wpm     2043 x .01 ms - low
	retlw   d'233'   ; 59.25 wpm     2025 x .01 ms - low
	retlw   d'216'   ; 59.75 wpm     2008 x .01 ms - low
	retlw   d'200'   ; 60.25 wpm     1992 x .01 ms - low
	retlw   d'183'   ; 60.75 wpm     1975 x .01 ms - low
	retlw   d'167'   ; 61.25 wpm     1959 x .01 ms - low
	retlw   d'151'   ; 61.75 wpm     1943 x .01 ms - low
	retlw   d'136'   ; 62.25 wpm     1928 x .01 ms - low
	retlw   d'120'   ; 62.75 wpm     1912 x .01 ms - low
	retlw   d'105'   ; 63.25 wpm     1897 x .01 ms - low
	retlw   d'90'    ; 63.75 wpm     1882 x .01 ms - low
	retlw   d'76'    ; 64.25 wpm     1868 x .01 ms - low
	retlw   d'61'    ; 64.75 wpm     1853 x .01 ms - low
	retlw   d'47'    ; 65.25 wpm     1839 x .01 ms - low
	retlw   d'33'    ; 65.75 wpm     1825 x .01 ms - low
	retlw   d'19'    ; 66.25 wpm     1811 x .01 ms - low
	retlw   d'6'     ; 66.75 wpm     1798 x .01 ms - low
	retlw   d'248'   ; 67.25 wpm     1784 x .01 ms - low
	retlw   d'235'   ; 67.75 wpm     1771 x .01 ms - low
	retlw   d'222'   ; 68.25 wpm     1758 x .01 ms - low
	retlw   d'209'   ; 68.75 wpm     1745 x .01 ms - low
	retlw   d'197'   ; 69.25 wpm     1733 x .01 ms - low
	retlw   d'184'   ; 69.75 wpm     1720 x .01 ms - low
	retlw   d'172'   ; 70.25 wpm     1708 x .01 ms - low
	retlw   d'160'   ; 70.75 wpm     1696 x .01 ms - low
	retlw   d'148'   ; 71.25 wpm     1684 x .01 ms - low
	retlw   d'136'   ; 71.75 wpm     1672 x .01 ms - low
	retlw   d'125'   ; 72.25 wpm     1661 x .01 ms - low
	retlw   d'113'   ; 72.75 wpm     1649 x .01 ms - low
	retlw   d'102'   ; 73.25 wpm     1638 x .01 ms - low
	retlw   d'91'    ; 73.75 wpm     1627 x .01 ms - low
	retlw   d'80'    ; 74.25 wpm     1616 x .01 ms - low
	retlw   d'69'    ; 74.75 wpm     1605 x .01 ms - low
	retlw   d'59'    ; 75.25 wpm     1595 x .01 ms - low
	retlw   d'48'    ; 75.75 wpm     1584 x .01 ms - low
	retlw   d'38'    ; 76.25 wpm     1574 x .01 ms - low
	retlw   d'28'    ; 76.75 wpm     1564 x .01 ms - low
	retlw   d'17'    ; 77.25 wpm     1553 x .01 ms - low
	retlw   d'7'     ; 77.75 wpm     1543 x .01 ms - low
	retlw   d'254'   ; 78.25 wpm     1534 x .01 ms - low
	retlw   d'244'   ; 78.75 wpm     1524 x .01 ms - low
	retlw   d'234'   ; 79.25 wpm     1514 x .01 ms - low
	retlw   d'225'   ; 79.75 wpm     1505 x .01 ms - low
	retlw   d'215'   ; 80.25 wpm     1495 x .01 ms - low
	retlw   d'206'   ; 80.75 wpm     1486 x .01 ms - low
	retlw   d'197'   ; 81.25 wpm     1477 x .01 ms - low
	retlw   d'188'   ; 81.75 wpm     1468 x .01 ms - low
	retlw   d'179'   ; 82.25 wpm     1459 x .01 ms - low
	retlw   d'170'   ; 82.75 wpm     1450 x .01 ms - low
	retlw   d'161'   ; 83.25 wpm     1441 x .01 ms - low
	retlw   d'153'   ; 83.75 wpm     1433 x .01 ms - low
	retlw   d'144'   ; 84.25 wpm     1424 x .01 ms - low
	retlw   d'136'   ; 84.75 wpm     1416 x .01 ms - low
	retlw   d'128'   ; 85.25 wpm     1408 x .01 ms - low
	retlw   d'119'   ; 85.75 wpm     1399 x .01 ms - low
	retlw   d'111'   ; 86.25 wpm     1391 x .01 ms - low
	retlw   d'103'   ; 86.75 wpm     1383 x .01 ms - low
	retlw   d'95'    ; 87.25 wpm     1375 x .01 ms - low
	retlw   d'88'    ; 87.75 wpm     1368 x .01 ms - low
	retlw   d'80'    ; 88.25 wpm     1360 x .01 ms - low
	retlw   d'72'    ; 88.75 wpm     1352 x .01 ms - low
	retlw   d'65'    ; 89.25 wpm     1345 x .01 ms - low
	retlw   d'57'    ; 89.75 wpm     1337 x .01 ms - low
	retlw   d'50'    ; 90.25 wpm     1330 x .01 ms - low
	retlw   d'42'    ; 90.75 wpm     1322 x .01 ms - low
	retlw   d'35'    ; 91.25 wpm     1315 x .01 ms - low
	retlw   d'28'    ; 91.75 wpm     1308 x .01 ms - low
	retlw   d'21'    ; 92.25 wpm     1301 x .01 ms - low
	retlw   d'14'    ; 92.75 wpm     1294 x .01 ms - low
	retlw   d'7'     ; 93.25 wpm     1287 x .01 ms - low
	retlw   d'0'     ; 93.75 wpm     1280 x .01 ms - low
	retlw   d'249'   ; 94.25 wpm     1273 x .01 ms - low
	retlw   d'242'   ; 94.75 wpm     1266 x .01 ms - low
	retlw   d'236'   ; 95.25 wpm     1260 x .01 ms - low
	retlw   d'229'   ; 95.75 wpm     1253 x .01 ms - low
	retlw   d'223'   ; 96.25 wpm     1247 x .01 ms - low
	retlw   d'216'   ; 96.75 wpm     1240 x .01 ms - low
	retlw   d'210'   ; 97.25 wpm     1234 x .01 ms - low
	retlw   d'204'   ; 97.75 wpm     1228 x .01 ms - low
	retlw   d'197'   ; 98.25 wpm     1221 x .01 ms - low
	retlw   d'191'   ; 98.75 wpm     1215 x .01 ms - low
	retlw   d'185'   ; 99.25 wpm     1209 x .01 ms - low
	retlw   d'179'   ; 99.75 wpm     1203 x .01 ms - low


; outdit - a routine to send a dit
ditlen  equ     d'80'  ;(80 x 1 ms = 80 ms)
outdit  movlw   ditlen  ;load up a 15 wpm dit length (1.2 / 15 = .08)
	movwf   temp2   ;store it in the second temp location
	goto    loop4   ;enter the dah routine


; outdah - a routine to send a dah
dahlen  equ     d'240' ;(240 x 1 ms = 240 ms)
outdah  movlw   dahlen  ;load up a 15 wpm dah length (3 x 1.2 / 15 = .24)
	movwf   temp2   ;store it in the second temp location
loop4   decfsz  temp2,1 ;decrement the dit length counter - is it 0 ?
	goto    beep4   ;no, start beeping
	;yes, end the dit
	goto    ditsp   ;with a dit space
beep4   bsf     PORTA,1 ; set piezo high
	call    halfcyc ;wait a half audio cycle
	bcf     PORTA,1 ; set piezo low
	call    halfcyc ;wait a half audio cycle
	goto    loop4   ;loop back until this dit length is complete


; ditsp - a routine to wait 1 dit space
; wdsp - a routine to wait 6 dit spaces + 1 from last dit or dah = 7 total 
; wdsp2 - a routine to wait 4 dit spaces + 3 from last char = 7 total 
; chsp - a routine to wait 2 dit spaces + 1 from last dit or dah = 3 total
ditsp   movlw   0x2     ;load up 1 dit space (add one due to loop contstruct)
	goto    loadet  ;branch in
wdsp    movlw   0x7     ;load up 6 dit spaces
	goto    loadet  ;branch in
wdsp2   movlw   0x5     ;load up 4 dit space (an additional delay after chsp)
	goto    loadet  ;branch into the routine with an alterante
chsp    movlw   0x3     ;load up 2 dit spaces
loadet  movwf   third   ;save it in third - a temp reg
decet   decfsz  third,1 ;decrement the dit space counter - is it 0 ?
	goto    ditsp2  ;no, start waiting
	goto    dun2    ;yes, end the dit
ditsp2  movlw   d'80'  ;load a 15 wpm dit length
	movwf   temp2   ;store it in the second temp location
loop6   decfsz  temp2,1 ;decrement the dit length counter - is it 0 ?
	goto    nobeep2 ;no, start waiting
	goto    decet   ;yes, check the dit space counter
nobeep2 call    halfcyc ;wait a half audio cycle
	call    halfcyc ;wait a half audio cycle
	goto    loop6   ;go back for another cycle?
dun2    return          ;end of a bunch of routines


; halfcyc - a routine to wait 1/2 of a sidetone cycle - in this case
;       that is 500 usec or 1/2 ms => 1000 Hz tone
;       at PIC clock = 4 MHz, 500 usec = 500 PIC instruction cycles
;       delay = 2 (call) + 6 (routine) + ((freq-1) x 3) loop delay
freq    equ     d'165' ; 8 + (164 x 3) = 500
halfcyc movlw   freq    ;1   1 load up the 1/2 cycle delay constant
	movwf   temp    ;1   2 store it in the temporary location
loopy   decfsz  temp,1  ;1     decrement the temp location - is it 0 ?
	goto    loopy   ;2     no, keep decrementing
	;yes, return to the calling routine
	return          ;2 500 end of half cycle routine


; strtbit - a routine to rotate a char left until a 1 appears
strtbit movlw   0x8     ;load up the maximum char size + 1
	movwf   size    ;save it temp
strbit2 decf    size,1  ;decrement the element counter
rotit   rlf     char,1  ;rotate the msb into carry
	btfss   STATUS,0 ;test the carry - 0=placeholder, 1=start bit
	goto    strbit2 ;it's a just a placeholder (0), keep shifting
	return          ;you're done - end of the startbit routine            


;        org     0x200


; playchr - a routine to play a character stored in w with Morse code
playchr call    numbers         ;lookup the value
	movwf   char            ;save it in char
getdig3 call    strtbit         ;rotate char left until start bit comes up
shout3  rlf     char,1          ;rotate the msb into carry
	btfsc   STATUS,0        ;test the carry - 0=dit, 1=dah
	goto    dahsnd3         ;it's a one, send a dah
	call    outdit          ;send a dit
	goto    sizdec          ;go check the size variable
dahsnd3 call    outdah          ;send the dah
sizdec  decfsz  size,1          ;decrement the char size counter, is it 0?        
	goto    shout3          ;no, go back for the next bit
	call    chsp            ;yes, send a character space and then exit
	return                  ;end of playchr routine


; init - the initialization power up entry point of the program
init    bsf     STATUS,5        ;set the bank bit to 1
	movlw   b'00000011'    ; all outs on PORTB xcept bits 1 & 0 (switches)
	movwf   TRISB           ; set the tri-state register
	movlw   b'00000100'    ; all outputs on PORTA, except bit 2
	movwf   TRISA           ; set the tri-state register
	movlw   b'00000000'    ; no option bits are relevant except PORTB p/u
	movwf   OPTION_REG      ; set the option register
	bcf     STATUS,5        ;set the bank bit to 0
	bcf     INTCON,GIE      ;disable interrupts
	call    blanko          ;initialize the LED display      
	call    blanko          ;   with 2 blank digits
	;11/26/2000 - blank them again, right digit had one seg on.
	call    blanko          ;initialize the LED display      
	call    blanko          ;   with 2 blank digits
	bsf     PORTB,3         ;turn off the 0 / .5 led
	movlw   d'10'          ;10 represents a dit longer than a 5 wpm dit
	movwf   lastdhi         ;initialize the last dit interval high < 5 wpm
	clrf    lastdlo         ; clear the low byte


mainlup ;check for input activity and branch if something happens
	btfss   PORTB,switch    ;has the Morse output switch been pressed?
	goto    morsout         ;yes, send the speed in Morse
	btfss   PORTB,1         ;has the Morse reading switch been pressed?
	goto    sendit          ;yes, send the reading in Morse
	btfsc   PORTA,in        ;no, check the input pin                
	goto    mainlup         ;no activity, keep looping


; timeit is a routine to measure the input, key down time
;       use 10 us chunks to 
timeit  clrf    highcb          ;1 clear the high counter byte
	clrf    lowcb           ;1       and the low counter byte, too
lstart  btfsc   PORTA,in        ;2   2 has the key been released?
	goto    bailo           ;      yes, bail out of the loop
	incfsz  lowcb,1         ;1   3  increment the low counter byte
	goto    evenit2         ;2   5  no rollover, even out the path delay
	incfsz  highcb,1        ;1   5  increment the high counter byte
	goto    evenit          ;2   7  no rollover, even out the path delay
nogood  clrf    highcb  ;the high order byte shouldn't rollovr
	clrf    lowcb
	goto    mainlup         ;bail back to the main loop.
evenit2 goto    evenit          ;2   7  waste 2 cycles
evenit  nop                     ;1   8 evens out the incfsz not needed for high byte
	goto    lstart          ;2  10 loop back for another 100 us
bailo   ;looping is over, now find the speed measured by the dit period


; testlen - a routine to compare the measured dit length against the length
;       table of half wpm values - then adjust the table position (of the
;       first table value to be less than the measured value) into the 
;       wpm value.  This is then converted to a 2 digit decimal and then
;       sent to the piezo.
; 1/28/00 - changed testlen to use lastdhi and lastdlo which are the last dit
;       length bytes - first the measured interval is divided by 2 and then
;       compared with the last dit intervals - if the measured interval
;       reading/2 is less than the last dit, assume it's also a dit, store it 
;       as the last dit interval and display the result - if the latest 
;       reading/2 is greater than the last dit, assume that it's a dah 
;       (3 x dit / 2 = 1.5 x dit) and don't display the reading.  This 
;       technique may be fooled by weighting or by a quickly changing speed 
;       (via pot turn?).
testlen 
; next, find the wpm equivalent of the measured interval        
	clrf    wpm             ;clear the table pointer
getdat  ;first, test wpm to see if it's less than 200
	movlw   d'182'          ;182 table entries
	subwf   wpm,0           ;is wpm less than 182?
	btfsc   STATUS,carry    ;is the result negative (carry = 0)?
	goto    mainlup         ;no, bail out, you are too high
	;yes, you are still below 182, keep going
	movf    wpm,0           ;load the wpm value into w
	call    length  ;get the high order byte of the length table value
	subwf   highcb,0  ;subtract it from the measured hi order length byte
	btfsc   STATUS,0        ;is the result negative (carry=0)?
	goto    seqlt           ;no, see if it's equal or less than measured
	incf    wpm,1           ;yes, increment the counter 
	goto    getdat          ;go back for another compare
seqlt   btfss   STATUS,2        ;a carry occurred, are the values equal? (Z=1)
	goto    display         ;no, enter the display routine
higheq  ;yes, the high order bytes are equal, check the low order bytes
	movf    wpm,0           ;load the pointer into w
	bsf     PCLATH,0        ;set pclath bit 0 for the page 1 table read
	call    length2         ;get the low order length table byte
	bcf     PCLATH,0        ;reset pclath bit 0 
	subwf   lowcb,0    ;subtract it from the measured lo order length byte
	btfsc   STATUS,0        ;is the result negative (carry=0)?
	goto    display      ;no, it's equal or less than measured, display it
	incf    wpm,1           ;yes, increment the table pointer and
	goto    getdat          ;then go back for another compare
	

; display - a routine to send the 2 (or 1) digit wpm value to the 
;       piezo output in Morse code at 15 wpm - changed display to
;       send the data to a pair of 7 segment LEDs, serially, LSD first
display bcf     STATUS,0        ;clear the carry bit
	bsf     PORTB,3 ;turn off the LED for sure ;)
	rrf     wpm,1   ;the table is less than the measure value, 
			;adjust wpm for display and load into w
	btfsc   STATUS,C        ;has the carry been set (.0 wpm flag?)
	bcf     PORTB,3 ;yes, turn on the LED
	;no, leave the LED off
	movlw   d'4'    ;increment the reading by 5 due to table start at 5 wpm
	addwf   wpm,0   ;add 5 to the reading and save in w
	call    bin2dec ;convert the binary number to 2 dec digits
	;first do the 1's digit
wuns    movf    ones,0  ;get the character
	call    out     ;send the digit to the LED
overtst movf    tens,1  ;test the tens digit
	btfsc   STATUS,2 ;skip playing the char if it is zero
	goto    blank   ;it is zero, send a blank digit to the LED
	movf    tens,0  ;it's not zero, get the digit
tensout call    out     ;send the digit 
	goto    mainlup ;return to the main loop
blank   movlw   b'00000000'    ;all zeroes is a blank display digit
	call    blanko  ;send the blank digit
	goto    mainlup ;return to the main loop

	
; morsout - a routine to send the last measured speed to the 
;       piezo in Morse code (at 15 WPM)
morsout movf    tens,0          ;load the tens digit
	btfsc   STATUS,zero     ;is the leading digit a 0?
	goto    vvuns           ;yes, don't send it, proceed to the ones digit
	call    playchr         ;play the character
vvuns   movf    ones,0          ;get the ones digit
	call    playchr         ;play the character
	goto    mainlup         ;return to the main loop


; sendit - send the 5 digits of the element count (from bin2de2) to the piezo
sendit  call    bin2de2         ;do the conversion from binary to bcd
	movf    R0,0            ;load up the highest digit
	btfsc   STATUS,zero     ;is the leading digit a 0?
	goto    midhi           ;yes, don't send it, just go to next digit
	call    maskns          ;send the highest digit to the piezo
midhi   swapf   R1,0            ;load up the middle byte & swap nibbles
	call    maskns          ;send the upper nibble of the middle byte
	movf    R1,0            ;load up the middle byte
	call    maskns          ;send the lower nibble of the middle byte
	swapf   R2,0            ;load up the low byte and swap nibbles
	call    maskns          ;send the upper nibble of the low byte
	movf    R2,0            ;load up the low byte
	call    maskns          ;send the lower nibble of the low byte
	goto    mainlup         ;re-enter the main loop


;maskns - mask and send the 4 lower bits of w to the piezo in Morse
maskns  andlw   b'00001111'     ;mask out the upper 4 bits
	;call    numbers         ;get the Morse equivalent of the hex number
	call    playchr         ;play this char
	return                  ;end of maskns


; bin2dec - a routine to convert a single binary byte into a 2 decimal digit 
;       equivalent.  The routine subtracts 10 from the binary byte until
;       a negative result occurs, keeping a count of the number of times 
;       that 10 is subtracted - then the last 10 is added back - the
;       counter for the 10s subtraction contains the tens decimal digit
;       equivalent of the binary number, the holder for the binary number
;       contains the ones decimal digit equivalent.
;   memory used (destroyed): tens, ones
;   bin2dec is entered with the binary number in W.
bin2dec clrf    tens    ;clear the tens holder
	movwf   ones    ;store the binary byte into ones
incre   incf    tens,1  ;bump the tens holder
	movlw   0x0A    ;load up a 10 into w
	subwf   ones,1  ;try subtracting 10 from the binary value
	btfsc   STATUS,0 ;keep going until result is negative (carry=0)
	goto    incre   ;it's positive, keep looping
	decf    tens,1  ;too far, give back the last increment
	movlw   0x0A    ;load up a 10 into w
	addwf   ones,1  ;"    "    add back the last subtraction
	;so now the 10's digit is in tens
	; and the 1's digit is in ones
	return          ;end of bin2dec rouitne


; bin2de2 - a routine to convert a two binary bytes into up to 5 decimal digit 
;       equivalent.  The routine is from Microchip app note: AN526
;   memory used (destroyed): R0, R1, R2, count, temp, hibyte, lobyte
;   bin2de2 uses the data in highcb and lowcb
bin2de2 movf    lowcb,0         ;load up the low byte
	movwf   lobyte          ;save it in the low scratch byte
	movf    highcb,0        ;load up the high byte
	movwf   hibyte          ;save it in the high scratch byte
	bcf     STATUS,0        ;clear the carry bit
	movlw   d'16'           ;load up 16 into
	movwf   count           ;       the bit counter
	clrf    R0              ;clear the bcd digit holders - highest digit
	clrf    R1              ;       middle 2 digits
	clrf    R2              ;       lowest 2 digits
	goto    loop16          ;branch over the adjdec routine
adjdec  movlw   R2              ;
	movwf   FSR             ;
	call    adjbcd          ;
	movlw   R1              ;
	movwf   FSR             ;
	call    adjbcd          ;
	movlw   R0              ;
	movwf   FSR             ;
	call    adjbcd          ;
loop16  rlf     lobyte,1        ;
	rlf     hibyte,1        ;
	rlf     R2,1            ;
	rlf     R1,1            ;
	rlf     R0,1            ;
	decfsz  count,1         ;
	goto    adjdec          ;
	return                  ;end of bin2dec rouitne

adjbcd  movlw   H'03'           ;
	addwf   INDF,0          ;
	movwf   temp            ;
	btfsc   temp,3          ;test if result > 7
	movwf   0               ;it is, save the result of the addition
	;it isn't, ignore the addition
	movlw   H'30'           ;
	addwf   0,0             ;
	movwf   temp            ;
	btfsc   temp,7          ;test if upper nibble result > 7
	movwf   0               ;save as msd
	retlw   0               ;end of adjbcd routine, clear w and return



; out - a routine to  convert the contents of w to segment data and then
;       send w bits out serially, SPI style to the HC595
blanko  movlw   b'00000000'    ;load up a blank digit
	goto    out2            ;display the blank
out     call    sevseg  ;convert the digit data in w to segment data
out2    bcf     PORTA,select    ;select the HC595s for data transfer
	movwf   temp    ;store the segment data temporarily
	movlw   8       ;setup for an 8 bit xfer
	movwf   temp2   ;store the count temp
	bcf     STATUS,carry    ;clear the carry bit
mor8    rlf     temp,1  ; rotate the segment data to shift a bit out
	btfss   STATUS,carry    ;is the bit a one or a zero
	goto    zerosnd ;send a zero
one     bsf     PORTA,datao     ;send a one
strb    bsf     PORTA,strobe    ;send strobe
	bcf     PORTA,strobe    ;clear the strobe
	decfsz  temp2,1         ;decrement the bit counter
	goto    mor8            ;it's not zero, keep going
	bsf     PORTA,select    ;deselect the 74HC595
	return                  ;end of the out routine
zerosnd bcf     PORTA,datao     ;send a zero
	goto    strb            ;strobe it in


	END     ;end of the speedf84.asm program


